@@ -64,6 +64,10 @@ class Full_Cone {
     friend class Collector<Integer>;
     
 public:
+    
+    int omp_start_level; // records the omp_get_level() when the computation is started
+                         // recorded at the start of the top cone constructor and the compute functions
+                         // compute and dualize_cone
     size_t dim;
     size_t level0_dim; // dim of cone in level 0 of the inhomogeneous case
     size_t module_rank;  // rank of solution module over level 0 monoid in the inhomogeneous case
@@ -80,11 +84,12 @@ public:
     bool deg1_extreme_rays;
     bool deg1_triangulation;
     bool deg1_hilbert_basis;
-    bool inhomogeneous; 
+    bool inhomogeneous;
     
     // control of what to compute
     bool do_triangulation;
     bool explicit_full_triang; // indicates whether full triangulation is asked for without default mode
+    bool explicit_h_vector; // to distinguish it from being set via default mode
     bool do_partial_triangulation;
     bool do_determinants;
     bool do_multiplicity;
@@ -97,13 +102,15 @@ public:
     bool do_excluded_faces;
     bool do_approximation;
     bool do_default_mode;
-	bool do_bottom_dec;
-	bool keep_order;
+    bool do_bottom_dec;
+    bool suppress_bottom_dec;
+    bool keep_order;
     bool do_class_group;
     bool do_module_gens_intcl;
     bool do_module_rank;
     bool do_cone_dec;
     bool stop_after_cone_dec;
+    bool do_hsop;
     
     bool do_extreme_rays;
     bool do_pointed;
@@ -126,9 +133,13 @@ public:
     vector<Integer> Sorting;
     mpq_class multiplicity;
     Matrix<Integer> Generators;
+    Matrix<Integer> ExtStrahl;
     vector<key_t> PermGens;  // stores the permutation of the generators created by sorting
-    vector<bool> Extreme_Rays;
+    vector<bool> Extreme_Rays_Ind;
     Matrix<Integer> Support_Hyperplanes;
+    Matrix<Integer> Subcone_Support_Hyperplanes; // used if *this computes elements in a subcone, for example in approximation
+    Matrix<Integer> Subcone_Equations;
+    vector<Integer> Subcone_Grading;
     size_t nrSupport_Hyperplanes;
     list<vector<Integer> > Hilbert_Basis;
     vector<Integer> Witness;    // for not integrally closed
@@ -138,15 +149,15 @@ public:
     size_t CandidatesSize;
     list<vector<Integer> > Deg1_Elements;
     HilbertSeries Hilbert_Series;
-    vector<long> gen_degrees;  // will contain the degrees of the generators
+    vector<Integer> gen_degrees;  // will contain the degrees of the generators
     Integer shift; // needed in the inhomogeneous case to make degrees positive
-    vector<long> gen_levels;  // will contain the levels of the generators (in the inhomogeneous case)
+    vector<Integer> gen_levels;  // will contain the levels of the generators (in the inhomogeneous case)
     size_t TriangulationBufferSize;          // number of elements in Triangulation, for efficiency
     list< SHORTSIMPLEX<Integer> > Triangulation;       // triangulation of cone
     list< SHORTSIMPLEX<Integer> > TriangulationBuffer; // simplices to evaluate
     list< SimplexEvaluator<Integer> > LargeSimplices; // Simplices for internal parallelization
     Integer detSum;                  // sum of the determinants of the simplices
-    list< STANLEYDATA<Integer> > StanleyDec; // Stanley decomposition
+    list< STANLEYDATA_int> StanleyDec; // Stanley decomposition
     vector<Integer> ClassGroup;  // the class group as a vector: ClassGroup[0]=its rank, then the orders of the finite cyclic summands
     
     Matrix<Integer> ProjToLevel0Quot;  // projection matrix onto quotient modulo level 0 sublattice    
@@ -166,6 +177,9 @@ public:
         size_t BornAt;                      // number of generator (in order of insertion) at which this hyperplane was added,, counting from 0
         size_t Ident;                      // unique number identifying the hyperplane (derived from HypCounter)
         size_t Mother;                     // Ident of positive mother if known, 0 if unknown
+        bool is_positive_on_all_original_gens;
+        bool is_negative_on_some_original_gen;
+        bool simplicial;                   // indicates whether facet is simplicial
     };
 
     list<FACETDATA> Facets;  // contains the data for Fourier-Motzkin and extension of triangulation
@@ -194,6 +208,7 @@ public:
     size_t store_level; // the level on which daughters will be stored  
     deque< list<vector<key_t> > > Pyramids;  //storage for pyramids
     deque<size_t> nrPyramids; // number of pyramids on the various levels
+    deque<bool> Pyramids_scrambled; // only used for mic
 
     // data that can be used to go out of build_cone and return later (not done at present)
     // but also useful at other places
@@ -216,6 +231,8 @@ public:
 #ifdef NMZ_MIC_OFFLOAD
     MicOffloader<long long> mic_offloader;
 #endif
+void try_offload_loc(long place,size_t max_level);
+
 
     // defining semiopen cones
     Matrix<Integer> ExcludedFaces;
@@ -231,8 +248,13 @@ public:
     
     size_t AdjustedReductionBound;
     
-    long approx_level;
     bool is_approximation;
+    bool is_global_approximation; // true if approximation is defined in Cone
+
+    vector<vector<key_t>> approx_points_keys;
+    Matrix<Integer> OriginalGenerators;
+
+    Integer VolumeBound; //used to stop computation of approximation if simplex of this has larger volume
 
 /* ---------------------------------------------------------------------------
  *              Private routines, used in the public routines
@@ -241,7 +263,7 @@ public:
     void number_hyperplane(FACETDATA& hyp, const size_t born_at, const size_t mother);
     bool is_hyperplane_included(FACETDATA& hyp);
     void add_hyperplane(const size_t& new_generator, const FACETDATA & positive,const FACETDATA & negative,
-                     list<FACETDATA>& NewHyps);
+                     list<FACETDATA>& NewHyps, bool known_to_be_simplicial);
     void extend_triangulation(const size_t& new_generator);
     void find_new_facets(const size_t& new_generator);
     void process_pyramids(const size_t new_generator,const bool recursive);
@@ -262,13 +284,14 @@ public:
     vector<key_t>  find_start_simplex() const;
     void store_key(const vector<key_t>&, const Integer& height, const Integer& mother_vol,
                                   list< SHORTSIMPLEX<Integer> >& Triangulation);
-	void find_bottom_facets();                                  
-    Matrix<Integer> latt_approx(); // makes a cone over a lattice polytope approximating "this"
+    void find_bottom_facets();                                  
+    vector<list<vector<Integer>>> latt_approx(); // makes a cone over a lattice polytope approximating "this"
     void convert_polyhedron_to_polytope();
     void compute_elements_via_approx(list<vector<Integer> >& elements_from_approx); // uses the approximation
-	void compute_deg1_elements_via_approx_global(); // deg 1 elements from the approximation
-    void compute_deg1_elements_via_approx_simplicial(const vector<key_t>& key); // the same for a simplicial subcone
-    void compute_sub_div_elements(const Matrix<Integer>& gens,list<vector<Integer> >& sub_div_elements); //computes subdividing elements via approximation
+    void compute_deg1_elements_via_approx_global(); // deg 1 elements from the approximation
+    void compute_deg1_elements_via_projection_simplicial(const vector<key_t>& key); // for a simplicial subcone by projecion
+    void compute_sub_div_elements(const Matrix<Integer>& gens,list<vector<Integer> >& sub_div_elements,
+    bool best_point=false); //computes subdividing elements via approximation
     void select_deg1_elements(const Full_Cone& C);
 //    void select_Hilbert_Basis(const Full_Cone& C); //experimental, unused
     
@@ -286,6 +309,7 @@ public:
     Matrix<Integer> select_matrix_from_list(const list<vector<Integer> >& S,vector<size_t>& selection);
 
     bool contains(const vector<Integer>& v);
+    bool subcone_contains(const vector<Integer>& v);
     bool contains(const Full_Cone& C);
     void extreme_rays_and_deg1_check();
     void find_grading();
@@ -319,9 +343,9 @@ public:
     void check_grading_after_dual_mode();
 
     void minimize_support_hyperplanes();   
-    void compute_extreme_rays();
-    void compute_extreme_rays_compare();
-    void compute_extreme_rays_rank();
+    void compute_extreme_rays(bool use_facets=false);
+    void compute_extreme_rays_compare(bool use_facets);
+    void compute_extreme_rays_rank(bool use_facets);
     void select_deg1_elements();
 
     void check_pointed();
@@ -338,6 +362,13 @@ public:
     void reset_tasks();
     void addMult(Integer& volume, const vector<key_t>& key, const int& tn); // multiplicity sum over thread tn
     
+    void check_simpliciality_hyperplane(const FACETDATA& hyp) const;
+    void set_simplicial(FACETDATA& hyp);
+    
+
+    void compute_hsop();
+    void heights(list<vector<key_t>>& facet_keys,list<pair<boost::dynamic_bitset<>,size_t>> faces, size_t index,vector<size_t>& ideal_heights, size_t max_dim);
+    
     void start_message();
     void end_message();
     
@@ -355,7 +386,7 @@ public:
  *                      Constructors
  *---------------------------------------------------------------------------
  */
-    Full_Cone(Matrix<Integer> M, bool do_make_prime=true);            //main constructor
+    Full_Cone(const Matrix<Integer>& M, bool do_make_prime=true);            //main constructor
     Full_Cone(Cone_Dual_Mode<Integer> &C);            // removes data from the argument!
     Full_Cone(Full_Cone<Integer>& C, const vector<key_t>& Key); // for pyramids
 
