@@ -27,8 +27,10 @@
 #include <set>
 #include <algorithm>
 #include <math.h>
+#include <iomanip>
 
 #include "libnormaliz/matrix.h"
+#include "libnormaliz/cone.h"
 #include "libnormaliz/vector_operations.h"
 #include "libnormaliz/normaliz_exception.h"
 #include "libnormaliz/sublattice_representation.h"
@@ -119,7 +121,7 @@ Matrix<Integer>::Matrix(const list< vector<Integer> >& new_elem){
 }
 
 //---------------------------------------------------------------------------
-
+/*
 template<typename Integer>
 void Matrix<Integer>::write(istream& in){
     size_t i,j;
@@ -129,31 +131,7 @@ void Matrix<Integer>::write(istream& in){
         }
     }
 }
-
-//---------------------------------------------------------------------------
-
-template<typename Integer>
-void Matrix<Integer>::write(size_t row, const vector<Integer>& data){
-    assert(row >= 0);
-    assert(row < nr); 
-    assert(nc == data.size());
-    
-    elem[row]=data;
-}
-
-//---------------------------------------------------------------------------
-
-template<typename Integer>
-void Matrix<Integer>::write(size_t row, const vector<int>& data){
-    assert(row >= 0);
-    assert(row < nr); 
-    assert(nc == data.size());
-
-    for (size_t i = 0; i < nc; i++) {
-        elem[row][i]=data[i];
-    }
-}
-
+*/
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -169,18 +147,6 @@ void Matrix<Integer>::write_column(size_t col, const vector<Integer>& data){
 
 //---------------------------------------------------------------------------
 
-template<typename Integer>
-void Matrix<Integer>::write(size_t row, size_t col, Integer data){
-    assert(row >= 0);
-    assert(row < nr); 
-    assert(col >= 0);
-    assert(col < nc); 
-
-    elem[row][col]=data;
-}
-
-//---------------------------------------------------------------------------
-
 template<typename Integer>
 void Matrix<Integer>::print(const string& name,const string& suffix) const{
     string file_name = name+"."+suffix;
@@ -204,9 +170,10 @@ void Matrix<Integer>::print_append(const string& name,const string& suffix) cons
 //---------------------------------------------------------------------------
 
 template<typename Integer>
-void Matrix<Integer>::print(ostream& out) const{
+void Matrix<Integer>::print(ostream& out, bool with_format) const{
     size_t i,j;
-    out<<nr<<endl<<nc<<endl;
+    if(with_format)
+        out<<nr<<endl<<nc<<endl;
     for (i = 0; i < nr; i++) {
         for (j = 0; j < nc; j++) {
             out<<elem[i][j]<<" ";
@@ -218,61 +185,24 @@ void Matrix<Integer>::print(ostream& out) const{
 //---------------------------------------------------------------------------
 
 template<typename Integer>
-void Matrix<Integer>::pretty_print(ostream& out, bool with_row_nr) const{    
-    size_t i,j,k;
+void Matrix<Integer>::pretty_print(ostream& out, bool with_row_nr) const{
+    if(nr>1000000 && !with_row_nr){
+        print(out,false);
+        return;
+    }
+    size_t i,j;
     vector<size_t> max_length = maximal_decimal_length_columnwise();
     size_t max_index_length = decimal_length(nr);
     for (i = 0; i < nr; i++) {
         if (with_row_nr) {
-            for (k= 0; k <= max_index_length - decimal_length(i); k++) {
-                out<<" ";
-            }
-            out << i << ": ";
+            out << std::setw(max_index_length+1) << i<< ": ";
         }
         for (j = 0; j < nc; j++) {
-            for (k= 0; k <= max_length[j] - decimal_length(elem[i][j]); k++) {
-                out<<" ";
-            }
-            out<<elem[i][j];
+            out << std::setw(max_length[j]+1) << elem[i][j];
         }
         out<<endl;
     }
 }
-//---------------------------------------------------------------------------
-
-
-template<typename Integer>
-void Matrix<Integer>::read() const{      //to overload for files
-    size_t i,j;
-    for(i=0; i<nr; i++){
-        cout << "\n" ;
-        for(j=0; j<nc; j++) {
-            cout << elem[i][j] << " ";
-        }
-    }
-}
-
-//---------------------------------------------------------------------------
-
-template<typename Integer>
-vector<Integer> Matrix<Integer>::read(size_t row) const{
-    assert(row >= 0);
-    assert(row < nr);
-
-    return elem[row];
-}
-
-//---------------------------------------------------------------------------
-
-template<typename Integer>
-Integer Matrix<Integer>::read (size_t row, size_t col) const{
-    assert(row >= 0);
-    assert(row < nr); 
-    assert(col >= 0);
-    assert(col < nc); 
-
-    return elem[row][col];
-}
 
 //---------------------------------------------------------------------------
 
@@ -290,6 +220,13 @@ size_t Matrix<Integer>::nr_of_columns () const{
 
 //---------------------------------------------------------------------------
 
+template<typename Integer>
+void Matrix<Integer>::set_nr_of_columns(size_t c){
+    nc=c;
+}
+
+//---------------------------------------------------------------------------
+
 template<typename Integer>
 void Matrix<Integer>::random (int mod) {
     size_t i,j;
@@ -463,12 +400,11 @@ vector<Integer> Matrix<Integer>::diagonal() const{
 
 template<typename Integer>
 size_t Matrix<Integer>::maximal_decimal_length() const{
-    size_t i,j,maxim=0;
-    for (i = 0; i <nr; i++) {
-        for (j = 0; j <nc; j++) {
-            maxim=max(maxim,decimal_length(elem[i][j]));
-        }
-    }
+    size_t i,maxim=0;
+    vector<size_t> maxim_col;
+    maxim_col=maximal_decimal_length_columnwise();
+    for (i = 0; i <nr; i++)
+        maxim=max(maxim,maxim_col[i]);
     return maxim;
 }
 
@@ -478,11 +414,21 @@ template<typename Integer>
 vector<size_t> Matrix<Integer>::maximal_decimal_length_columnwise() const{
     size_t i,j=0;
     vector<size_t> maxim(nc,0);
+    vector<Integer> pos_max(nc,0), neg_max(nc,0);
     for (i = 0; i <nr; i++) {
         for (j = 0; j <nc; j++) {
-            maxim[j]=max(maxim[j],decimal_length(elem[i][j]));
+            // maxim[j]=max(maxim[j],decimal_length(elem[i][j]));
+            if(elem[i][j]<0){
+                if(elem[i][j]<neg_max[j])
+                    neg_max[j]=elem[i][j];
+                continue;
+            }
+            if(elem[i][j]>pos_max[j])
+                pos_max[j]=elem[i][j];
         }
     }
+    for(size_t j=0;j<nc;++j)
+        maxim[j]=max(decimal_length(neg_max[j]),decimal_length(pos_max[j]));
     return maxim;
 }
 
@@ -492,9 +438,10 @@ template<typename Integer>
 void Matrix<Integer>::append(const Matrix<Integer>& M) {
     assert (nc == M.nc);
     elem.reserve(nr+M.nr);
-    for (size_t i=0; i<M.nr; i++) {
+    /* for (size_t i=0; i<M.nr; i++) {
         elem.push_back(M.elem[i]);
-    }
+    }*/
+    elem.insert(elem.end(),M.elem.begin(),M.elem.end());
     nr += M.nr;
 }
 
@@ -549,9 +496,19 @@ void Matrix<Integer>::remove_row(const vector<Integer>& row) {
 //---------------------------------------------------------------------------
 
 template<typename Integer>
-void Matrix<Integer>::remove_duplicate_and_zero_rows() {
+void Matrix<Integer>::remove_row(const size_t index) {
+    assert(index<nr);
+    nr--;
+    elem.erase(elem.begin()+(index));
+}
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+vector<size_t> Matrix<Integer>::remove_duplicate_and_zero_rows() {
     bool remove_some = false;
     vector<bool> key(nr, true);
+    vector<size_t> original_row;
 
     set<vector<Integer> > SortedRows;
     SortedRows.insert( vector<Integer>(nc,0) );
@@ -562,8 +519,34 @@ void Matrix<Integer>::remove_duplicate_and_zero_rows() {
             key[i] = false;
             remove_some = true;
         }
-        else
+        else{
             SortedRows.insert(found,elem[i]);
+            original_row.push_back(i);
+        }
+    }
+
+    if (remove_some) {
+        *this = submatrix(key);
+    }
+    return original_row;
+}
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+void Matrix<Integer>::remove_duplicate(const Matrix<Integer>& M) {
+    bool remove_some = false;
+    vector<bool> key(nr, true);
+
+    // TODO more efficient! sorted rows
+    for (size_t i = 0; i<nr; i++) {
+        for (size_t j=0;j<M.nr_of_rows();j++){
+            if (elem[i]==M[j]){
+                remove_some=true;
+                key[i]=false;
+                break;
+            }
+        }
     }
 
     if (remove_some) {
@@ -594,38 +577,19 @@ template<typename Integer>
 Matrix<Integer> Matrix<Integer>::multiplication(const Matrix<Integer>& A) const{
     assert (nc == A.nr);
 
-    Matrix<Integer> B(nr,A.nc,0);  //initialized with 0
+    Matrix<Integer> Atrans=A.transpose();
+    Matrix<Integer> B(nr,A.nc);  //initialized with 0
     size_t i,j,k;
     for(i=0; i<B.nr;i++){
         for(j=0; j<B.nc; j++){
             for(k=0; k<nc; k++){
-                B.elem[i][j]=B.elem[i][j]+elem[i][k]*A.elem[k][j];
+                B[i][j]=v_scalar_product(elem[i],Atrans[j]);
             }
         }
     }
     return B;
 }
 
-//---------------------------------------------------------------------------
-
-template<typename Integer>
-Matrix<Integer> Matrix<Integer>::multiplication_cut(const Matrix<Integer>& A, const size_t& c) const{
-    assert (nc == A.nr);
-    assert(c<= A.nc);
-
-    Matrix<Integer> B(nr,c,0);  //initialized with 0
-    size_t i,j,k;
-    for(i=0; i<B.nr;i++){
-        for(j=0; j<c; j++){
-            for(k=0; k<nc; k++){
-                B.elem[i][j]=B.elem[i][j]+elem[i][k]*A.elem[k][j];
-            }
-        }
-    }
-    return B;
-}
-
-
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -1917,6 +1881,66 @@ size_t Matrix<Integer>::row_echelon(){
     return rk;
 }
 
+//-----------------------------------------------------------
+//
+// variants for floating point
+//
+//-----------------------------------------------------------
+
+template<>
+long Matrix<nmz_float>::pivot_column(size_t row,size_t col){
+    
+    size_t i;
+    long j=-1;
+    nmz_float help=0;
+
+    for (i = row; i < nr; i++) {
+        if (Iabs(elem[i][col])>nmz_epsilon) {
+            if ((help==0)||(Iabs(elem[i][col])>help)) {
+                help=Iabs(elem[i][col]);
+                j=i;
+            }        }
+    }
+
+    return j;
+}
+
+template<>
+size_t Matrix<nmz_float>::row_echelon_inner_elem(bool& success){
+
+    size_t pc=0;
+    long piv=0, rk=0;
+
+    if(nr==0)
+        return 0;
+    
+    for (rk = 0; rk < (long) nr; rk++){
+        for(;pc<nc;pc++){
+            piv=pivot_column(rk,pc);
+            if(piv>=0)
+                break;
+        }
+        if(pc==nc)
+            break;
+            
+        exchange_rows (rk,piv);
+        reduce_row(rk,pc);
+    }
+    
+    success=true;                
+    return rk;
+}
+
+
+template<>
+size_t Matrix<nmz_float>::row_echelon(){
+
+    size_t rk;
+    bool dummy;
+    rk=row_echelon_inner_elem(dummy);
+    Shrink_nr_rows(rk);
+    return rk;
+}
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -2123,35 +2147,35 @@ void mpz_submatrix_trans(Matrix<mpz_class>& sub, const Matrix<Integer>& mother,
 template<typename Integer>
 vector<key_t> Matrix<Integer>::perm_sort_by_degree(const vector<key_t>& key, const vector<Integer>& grading, bool computed) const{
 
-	list<vector<Integer>> rowList;
-	vector<Integer> v;
+            list<vector<Integer>> rowList;
+            vector<Integer> v;
 
-	v.resize(nc+2);
-	unsigned long i,j;
-	
-	for (i=0;i<key.size();i++){
-		if (computed){
-		v[0]=v_scalar_product((*this).elem[key[i]],grading);
-		} else{
-			v[0]=0;
-			for (j=0;j<nc;j++) v[0]+=Iabs((*this).elem[key[i]][j]);
-		}
-		for (j=0;j<nc;j++){
-			v[j+1] = (*this).elem[key[i]][j];
-		}
-		v[nc+1] = key[i]; // position of row
-		rowList.push_back(v);
-	}
-	rowList.sort();
-	vector<key_t> perm;
-	perm.resize(key.size());
-	i=0;
-	for (typename list< vector<Integer> >::const_iterator it = rowList.begin();it!=rowList.end();++it){
-		perm[i]=convertTo<long>((*it)[nc+1]);
-		i++;
-	}
-	return perm;
-}
+            v.resize(nc+2);
+            unsigned long i,j;
+            
+            for (i=0;i<key.size();i++){
+                    if (computed){
+                    v[0]=v_scalar_product((*this).elem[key[i]],grading);
+                    } else{
+                            v[0]=0;
+                            for (j=0;j<nc;j++) v[0]+=Iabs((*this).elem[key[i]][j]);
+                    }
+                    for (j=0;j<nc;j++){
+                            v[j+1] = (*this).elem[key[i]][j];
+                    }
+                    v[nc+1] = key[i]; // position of row
+                    rowList.push_back(v);
+            }
+            rowList.sort();
+            vector<key_t> perm;
+            perm.resize(key.size());
+            i=0;
+            for (typename list< vector<Integer> >::const_iterator it = rowList.begin();it!=rowList.end();++it){
+                    perm[i]=convertTo<long>((*it)[nc+1]);
+                    i++;
+            }
+            return perm;
+    }
 
 //---------------------------------------------------------------------------
 
@@ -2336,6 +2360,9 @@ size_t Matrix<Integer>::extreme_points_first(const vector<Integer> norm){
     size_t no_success=0;
     // size_t nr_attempt=0;
     while(true){
+        
+        INTERRUPT_COMPUTATION_BY_EXCEPTION
+        
         // nr_attempt++; cout << nr_attempt << endl;
         vector<long long> L=v_random<long long>(nc,10);
         vector<key_t> max_min_ind;
@@ -2391,4 +2418,158 @@ void Matrix<Integer>::Shrink_nr_rows(size_t new_nr_rows){
     elem.resize(nr);
 }
 
+template<typename Integer>
+Matrix<Integer>  readMatrix(const string project){
+// reads one matrix from file with name project
+// format: nr of rows, nr of colimns, entries
+// all separated by white space
+    
+    string name_in=project;
+    const char* file_in=name_in.c_str();
+    ifstream in;
+    in.open(file_in,ifstream::in);
+    if (in.is_open()==false){
+        cerr << "Cannot find input file" << endl;
+        throw BadInputException("readMatrix cannot find file");
+    }
+    
+    int nrows,ncols;
+    in >> nrows;
+    in >> ncols;
+    
+    if(nrows==0 || ncols==0){
+        cerr << "Matrix empty" << endl;
+        throw BadInputException("readMatrix finds matrix empty");
+    }
+    
+    
+    int i,j,entry;
+    Matrix<Integer> result(nrows,ncols);
+    
+    for(i=0;i<nrows;++i)
+        for(j=0;j<ncols;++j){
+            in >> entry;
+            result[i][j]=entry;
+        }
+    return result;
+}
+
+//---------------------------------------------------------------------------
+// version with full number of points
+// and search for optimal point
+
+template<typename Integer>
+vector<Integer> Matrix<Integer>::optimal_subdivision_point() const{
+
+        return optimal_subdivision_point_inner();
+}
+
+// In mpz_class we first try machine integer
+template<>
+vector<mpz_class> Matrix<mpz_class>::optimal_subdivision_point() const{
+
+    try {
+        Matrix<MachineInteger> GensMI;
+        convert(GensMI,*this);
+        vector<MachineInteger> PMI=GensMI.optimal_subdivision_point_inner();
+        vector<mpz_class>  P;
+        convert(P,PMI);
+        return P;
+    } catch(const ArithmeticException& e) {
+        return optimal_subdivision_point_inner();
+    }
+}
+
+// version with a single point, only top of the search polytope
+// After 2 attempts without improvement, g raised to opt_value-1
+template<typename Integer>
+vector<Integer> Matrix<Integer>::optimal_subdivision_point_inner() const{
+// returns empty vector if simplex cannot be subdivided with smaller detsum
+      
+    // cout << "==================" << endl;
+  
+    assert(nr>0);
+    assert(nr==nc);
+      
+    vector<Integer> opt_point;
+  
+    vector<Integer> N = find_linear_form();
+    assert(N.size()==nr);
+    Integer G=v_scalar_product(N,elem[0]);
+    if(G<=1)
+        return opt_point;
+    Matrix<Integer> Supp;
+    Integer V;
+    vector<key_t> dummy(nr);
+    for(size_t i=0;i<nr;++i)
+        dummy[i]=i;
+    simplex_data(dummy, Supp,V,true);
+    Integer MinusOne=-1;
+    vector<Integer> MinusN(N);
+    v_scalar_multiplication(MinusN,MinusOne);
+    Supp.append(MinusN);
+    Supp.resize_columns(nr+1);
+    Supp.exchange_columns(0,nc); // grading to the front!
+      
+    Integer opt_value=G;
+    Integer empty_value=0;
+    Integer g=G-1;
+      
+    Integer den=2;
+      
+    vector<Integer> Zero(nr+1); // the excluded vector
+    Zero[0]=1;
+
+    // Incidence matrix for projectand lift    
+    vector<boost::dynamic_bitset<> > Ind(nr+1);
+    for(size_t i=0;i<nr+1;++i){
+        Ind[i].resize(nc+1);
+        for(size_t j=0;j<nc+1;++j)
+            Ind[i][j]=true;
+        Ind[i][i]=false;
+    }
+    
+    // cout << "==============================" << endl;
+    
+    size_t nothing_found=0;
+    while(true){
+        vector<Integer> SubDiv;
+        // cout << "Opt " << opt_value << " test " << g << " empty " << empty_value << " nothing "  << nothing_found << endl;
+        Supp[nr][0]=g;  // the degree at which we cut the simplex1;
+        ProjectAndLift<Integer,Integer> PL(Supp,Ind,nr+1);
+        PL.set_excluded_point(Zero);
+        PL.set_verbose(false);
+        PL.compute(false); // only a single point
+        PL.put_single_point_into(SubDiv);
+        if(SubDiv.size()==0){ // no point found
+            nothing_found++;
+            if(g==opt_value-1)
+                return opt_point; // optimal point found (or nothing found)
+            empty_value=g;
+            if(nothing_found<1) // can't be true if "1" is not raised to a higher value
+                g=empty_value+1+(den-1)*(opt_value-empty_value-2)/den;
+            else
+                g=opt_value-1;
+            den*=2;    // not used in the present setting (see above)      
+        }
+        else{ // point found
+            nothing_found=0;
+            den=2; // back to start value
+            opt_point=SubDiv;
+            std::swap(opt_point[0],opt_point[nc]);
+            opt_point.resize(nc);
+            if(opt_value==empty_value+1)
+                return opt_point;
+            opt_value=v_scalar_product(opt_point,N);
+            g=empty_value+1+(opt_value-empty_value-2)/2;
+        }
+    }
+}
+
+#ifndef NMZ_MIC_OFFLOAD  //offload with long is not supported
+template Matrix<long>  readMatrix(const string project);
+#endif // NMZ_MIC_OFFLOAD
+template Matrix<long long>  readMatrix(const string project);
+template Matrix<mpz_class>  readMatrix(const string project);
+
 }  // namespace
