@@ -29,6 +29,7 @@
 #include <iostream>
 #include <set>
 #include <deque>
+#include <csignal>
 
 #include <time.h>
 
@@ -203,14 +204,14 @@ Integer SimplexEvaluator<Integer>::start_evaluation(SHORTSIMPLEX<Integer>& s, Co
     //degrees of the generators according to the Grading of C
     if(C.isComputed(ConeProperty::Grading))
         for (i=0; i<dim; i++)
-            gen_degrees[i] = C.gen_degrees[key[i]];
+            gen_degrees[i] = convertTo<long>(C.gen_degrees[key[i]]);
             
     nr_level0_gens=0;
     level0_gen_degrees.clear();
     
     if(C.inhomogeneous){
         for (i=0; i<dim; i++){
-            gen_levels[i] = C.gen_levels[key[i]];
+            gen_levels[i] = convertTo<long>(C.gen_levels[key[i]]);
             if(gen_levels[i]==0){
                 nr_level0_gens++;
                 level0_gen_degrees.push_back(gen_degrees[i]);
@@ -434,10 +435,10 @@ void SimplexEvaluator<Integer>::take_care_of_0vector(Collector<Integer>& Coll){
         prepare_inclusion_exclusion_simpl(Deg0_offset, Coll);
 
     if(C_ptr->do_Stanley_dec){                          // prepare space for Stanley dec
-        STANLEYDATA<Integer> SimplStanley;         // key + matrix of offsets
+        STANLEYDATA_int SimplStanley;         // key + matrix of offsets
         SimplStanley.key=key;
         Matrix<Integer> offsets(convertTo<long>(volume),dim);  // volume rows, dim columns
-        SimplStanley.offsets=offsets;
+        convert(SimplStanley.offsets,offsets);
         #pragma omp critical(STANLEY)
         {
         C_ptr->StanleyDec.push_back(SimplStanley);      // extend the Stanley dec by a new matrix
@@ -445,7 +446,7 @@ void SimplexEvaluator<Integer>::take_care_of_0vector(Collector<Integer>& Coll){
         }
         for(i=0;i<dim;++i)                   // the first vector is 0+offset
             if(Excluded[i])
-                (*StanleyMat)[0][i]=volume;
+                (*StanleyMat)[0][i]=convertTo<long>(volume);
     }
 
     StanIndex=1;  // counts the number of components in the Stanley dec. Vector at 0 already filled if necessary
@@ -490,8 +491,12 @@ void SimplexEvaluator<Integer>::transform_to_global(const vector<Integer>& eleme
 
 template<typename Integer>
 void SimplexEvaluator<Integer>::evaluate_element(const vector<Integer>& element, Collector<Integer>& Coll){
+    
+    INTERRUPT_COMPUTATION_BY_EXCEPTION
+    
+    // now the vector in par has been produced and is in element    
+    // DON'T FORGET: THE VECTOR PRODUCED IS THE "REAL" VECTOR*VOLUME !!
 
-    // now we create and evaluate the points in par
     Integer norm;
     Integer normG;
     size_t i;
@@ -501,7 +506,7 @@ void SimplexEvaluator<Integer>::evaluate_element(const vector<Integer>& element,
     if(C.is_approximation && C.do_Hilbert_basis){
         vector<Integer> help(dim);
         transform_to_global(element,help);
-        if(!C.contains(help))
+        if(!C.subcone_contains(help)) // here we are abusing the support hyperplanes of the approximated cone !
             return;
         /* #pragma omp atomic
         NrCand++;*/
@@ -513,12 +518,6 @@ void SimplexEvaluator<Integer>::evaluate_element(const vector<Integer>& element,
     
     }
 
-    typename list <vector <Integer> >::iterator c;
-    
-    // now the vector in par has been produced and is in element
-    
-    // DON'T FORGET: THE VECTOR PRODUCED IS THE "REAL" VECTOR*VOLUME !!
-
     norm=0; // norm is just the sum of coefficients, = volume*degree if homogenous
             // it is used to sort the Hilbert basis candidates
     normG = 0;  // the degree according to the grading
@@ -573,10 +572,10 @@ void SimplexEvaluator<Integer>::evaluate_element(const vector<Integer>& element,
     }
 
     if(C.do_Stanley_dec){
-        (*StanleyMat)[StanIndex]=element;
+        convert((*StanleyMat)[StanIndex],element);
         for(i=0;i<dim;i++)
             if(Excluded[i]&&element[i]==0)
-                (*StanleyMat)[StanIndex][i]+=volume;
+                (*StanleyMat)[StanIndex][i]+=convertTo<long>(volume);
         StanIndex++;
     }
 
@@ -594,6 +593,9 @@ void SimplexEvaluator<Integer>::evaluate_element(const vector<Integer>& element,
     if(C.do_deg1_elements && normG==volume && !isDuplicate(element)) {
         vector<Integer> help(dim);
         transform_to_global(element,help);
+        if((C.is_approximation || C.is_global_approximation) && !C.subcone_contains(help)){
+            return;
+        }
         Coll.Deg1_Elements.push_back(help);
         Coll.collected_elements_size++;
     }
@@ -704,7 +706,11 @@ void SimplexEvaluator<Integer>::conclude_evaluation(Collector<Integer>& Coll) {
 //---------------------------------------------------------------------------
 
 
-const long SimplexParallelEvaluationBound=10000000; // larger simplices are evaluated by parallel threads
+const long SimplexParallelEvaluationBound=100000000; // simplices larger than this bound/10 
+                 //are evaluated by parallel threads
+                 // simplices larger than this bound  || (this bound/10 && Hilbert basis)
+                 // are tried for subdivision
+
 
 //---------------------------------------------------------------------------
 
@@ -721,8 +727,8 @@ bool SimplexEvaluator<Integer>::evaluate(SHORTSIMPLEX<Integer>& s) {
     if(C_ptr->do_cone_dec)
         s.Excluded=Excluded;
     // large simplicies to be postponed for parallel evaluation
-    if ( (volume > SimplexParallelEvaluationBound ||
-           (volume > SimplexParallelEvaluationBound/10 && C_ptr->do_Hilbert_basis) )
+    if ( volume > SimplexParallelEvaluationBound/10
+           // || (volume > SimplexParallelEvaluationBound/10 && C_ptr->do_Hilbert_basis) )
        && !C_ptr->do_Stanley_dec){ //&& omp_get_max_threads()>1)
         return false;        
     }
@@ -938,40 +944,48 @@ void SimplexEvaluator<Integer>::collect_vectors(){
 /* evaluates a simplex in parallel threads */
 template<typename Integer>
 void SimplexEvaluator<Integer>::Simplex_parallel_evaluation(){
+    
+    /* Generators.pretty_print(cout);
+    cout << "==========================" << endl; */
 
     if(C_ptr->verbose){
         verboseOutput() << "simplex volume " << volume << endl;
     }
-    if (C_ptr->use_bottom_points && volume >= SimplexParallelEvaluationBound
-        && C_ptr->approx_level == 1
+
+    if (C_ptr->use_bottom_points && (volume >= SimplexParallelEvaluationBound 
+        || (volume > SimplexParallelEvaluationBound/10 && C_ptr->do_Hilbert_basis) ) 
         && (!C_ptr->deg1_triangulation || !C_ptr->isComputed(ConeProperty::Grading)))
-    {
+    {  // try subdivision
 
         Full_Cone<Integer>& C = *C_ptr;
         
+        assert(C.omp_start_level==omp_get_level()); // make sure that we are on the lowest parallelization level
+        
         if (C_ptr->verbose) {
-				verboseOutput() << "**************************************************" << endl;
-				verboseOutput() << "Try to decompose the simplex into smaller simplices." << endl;
-		}
+            verboseOutput() << "**************************************************" << endl;
+            verboseOutput() << "Try to decompose the simplex into smaller simplices." << endl;
+        }
 
         for (size_t i=0; i<dim; ++i)
             Generators[i] = C.Generators[key[i]];
 
         list< vector<Integer> > new_points;
         time_t start,end;
-		time (&start);
-#ifndef NMZ_SCIP
-        C.compute_sub_div_elements(Generators, new_points);
-        //cout << "Found "<< new_points.size() << " bottom candidates via approximation" << endl;
-       
-#endif
-		bottom_points(new_points, Generators,C.Grading,C.approx_level,0);
+        time (&start);
+        void (*prev_handler)(int);
+        prev_handler = signal (SIGINT, SIG_IGN); // we don't want to set a new handler here
+        signal (SIGINT, prev_handler);
+    
+        bottom_points(new_points, Generators,volume);
+        
+        signal(SIGINT, prev_handler);
+        
         time (&end);
-		double dif = difftime (end,start);
+        double dif = difftime (end,start);
 
-		if (C_ptr->verbose) {
-				verboseOutput() << "Bottom points took " << dif << " sec " <<endl;
-		}
+        if (C_ptr->verbose) {
+            verboseOutput() << "Bottom points took " << dif << " sec " <<endl;
+        }
 
         // cout << new_points.size() << " new points " << endl << new_points << endl;
         if (!new_points.empty()) {
@@ -979,9 +993,8 @@ void SimplexEvaluator<Integer>::Simplex_parallel_evaluation(){
             // add new_points to the Top_Cone generators
             int nr_new_points = new_points.size();
             int nr_old_gen = C.nr_gen;
-            C.add_generators(Matrix<Integer>(new_points));
-            //cout << "generators: " << endl;
-            //C.Generators.pretty_print(cout);
+            Matrix<Integer> new_points_mat(new_points);
+            C.add_generators(new_points_mat);
             // remove this simplex from det_sum and multiplicity
             addMult(-volume,C.Results[0]);
             // delete this large simplex
@@ -996,7 +1009,11 @@ void SimplexEvaluator<Integer>::Simplex_parallel_evaluation(){
                 }
             }
 
-            // create subcone key
+            // create generators for bottom decomposition
+            // we start with the extreme rays of the recession cone
+            Matrix<Integer> BotGens=Generators;
+            BotGens.append_column( vector<Integer>(dim, 0) );
+            // now the polyhedron            
             vector<key_t> subcone_key(C.dim + nr_new_points);
             for (size_t i=0; i<C.dim; ++i) {
                 subcone_key[i] = key[i];
@@ -1004,71 +1021,50 @@ void SimplexEvaluator<Integer>::Simplex_parallel_evaluation(){
             for (int i=0; i<nr_new_points; ++i) {
                 subcone_key[C.dim + i] = nr_old_gen + i;
             }
-
-            //compute optimal triangulation
             Matrix<Integer> polytope_gens(C.Generators.submatrix(subcone_key));
-            key_t zero_point = polytope_gens.nr_of_rows();
-            polytope_gens.append( vector<Integer>(dim, 0) );
-            polytope_gens.append_column( vector<Integer>(zero_point+1, 1) );
-            Full_Cone<Integer> bottom_polytope(polytope_gens);
-            bottom_polytope.keep_order = true;
-            bottom_polytope.keep_triangulation = true;
-            //bottom_polytope.do_all_hyperplanes = false;
-            if (C_ptr->verbose) {
-				verboseOutput() << "Computing triangulation of bottom polytope... " << flush;
-			}
-            bool verbtmp = C_ptr->verbose;
-            C_ptr->verbose = false;
-			time (&start);
-            bottom_polytope.compute();
-            time (&end);
-			dif = difftime (end,start);
-            C_ptr->verbose = verbtmp;
-            if (C_ptr->verbose) {
-				verboseOutput() << "done." << endl;
-				verboseOutput() << "Computing triangulation took " << dif << " sec" << endl;
-			}
-            assert(bottom_polytope.isComputed(ConeProperty::Triangulation));
+            polytope_gens.append_column( vector<Integer>(polytope_gens.nr_of_rows(), 1) );
+            BotGens.append(polytope_gens);
 
-            // extract bottom triangulation
-            list< SHORTSIMPLEX<Integer> > bottom_triang;
-            typename list< SHORTSIMPLEX<Integer> >::iterator bottom_it;
-            bottom_it = bottom_polytope.Triangulation.begin();
-            typename list< SHORTSIMPLEX<Integer> >::const_iterator bottom_end;
-            bottom_end = bottom_polytope.Triangulation.end();
-            SHORTSIMPLEX<Integer> new_simplex;
-            new_simplex.key = vector<key_t>(dim);
-            key_t i;
+            //compute bottom decomposition
+            Full_Cone<Integer> bottom_polytope(BotGens);
+            bottom_polytope.keep_order = true;
+            // bottom_polytope.verbose=true;
             if (C_ptr->verbose) {
-				verboseOutput() << "Extracting and evaluating triangulation from bottom polytope..." << endl;
-			}
- 
-            for (; bottom_it != bottom_end; ++bottom_it) {
-                sort(bottom_it->key.begin(), bottom_it->key.end());
-                //cout << "org " << bottom_it->key;
-                // we assume sorted keys
-                if (bottom_it->key.back() == zero_point) {
-                    for (i = 0; i < dim; ++i)
-                        new_simplex.key[i] = subcone_key[bottom_it->key[i]];
-                    new_simplex.height = bottom_it->vol; // best replacement for height
-                    new_simplex.vol    = bottom_it->vol;
-                    //cout << "new " << new_simplex.key;
-                    C.TriangulationBuffer.push_back(new_simplex);
-                    C.TriangulationBufferSize++;
+                verboseOutput() << "Computing bottom decomposition ... " << flush;
+            }
+            time (&start);
+            bottom_polytope.dualize_cone(false);
+            time (&end);
+            dif = difftime (end,start);
+            if (C_ptr->verbose) {
+                verboseOutput() << "done." << endl;
+                verboseOutput() << "Bottom decomposition took " << dif << " sec" << endl;
+            }
+            assert(bottom_polytope.isComputed(ConeProperty::SupportHyperplanes));
+
+            // extract bottom decomposition
+            for(size_t i=0;i<bottom_polytope.Support_Hyperplanes.nr_of_rows();++i){
+                
+                INTERRUPT_COMPUTATION_BY_EXCEPTION
+                
+                if(bottom_polytope.Support_Hyperplanes[i][dim]>=0) // not a bottom facet
+                    continue;
+                vector<key_t>bottom_key;
+                for(size_t j=0;j<polytope_gens.nr_of_rows();++j){
+                    if(v_scalar_product(polytope_gens[j],bottom_polytope.Support_Hyperplanes[i])==0)
+                        bottom_key.push_back(subcone_key[j]);                    
                 }
+                C.Pyramids[0].push_back(bottom_key);
+                C.nrPyramids[0]++;
             }
 
-            // evaluate created triangulation
-            if (C.check_evaluation_buffer())
-                C.evaluate_triangulation();
-
             if (C_ptr->verbose) {
-				verboseOutput() << "**************************************************" << endl;
-			}
+                verboseOutput() << "**************************************************" << endl;
+            }
 
             return;
         }
-    }
+    } // end subdivision
 
     take_care_of_0vector(C_ptr->Results[0]);
 
@@ -1289,6 +1285,11 @@ vector<key_t> SimplexEvaluator<Integer>::get_key(){
     return key;
 }
 
+template<typename Integer>
+Integer SimplexEvaluator<Integer>::get_volume(){
+    return volume;
+}
+
 // Collector
 
 template<typename Integer>
@@ -1306,7 +1307,8 @@ Collector<Integer>::Collector(Full_Cone<Integer>& fc):
     size_t hv_max=0;
     if (C_ptr->do_h_vector) {
         // we need the generators to be sorted by degree
-        hv_max = C_ptr->gen_degrees[C_ptr->nr_gen-1] * C_ptr->dim;
+        long max_degree=convertTo<long>(C_ptr->gen_degrees[C_ptr->nr_gen-1]);
+        hv_max = max_degree * C_ptr->dim;
         if (hv_max > 1000000) {
             throw BadInputException("Generator degrees are too huge, h-vector would contain more than 10^6 entires.");
         }
