@@ -138,6 +138,214 @@ Integer v_scalar_product(const vector<Integer>& av,const vector<Integer>& bv){
 
 //---------------------------------------------------------------------------
 
+template<typename Integer>
+bool v_scalar_product_nonnegative(const vector<Integer>& av,const vector<Integer>& bv){
+    //loop stretching ; brings some small speed improvement
+
+    Integer ans = 0;
+    size_t i,n=av.size();
+
+
+#if 0 // #ifdef __MIC__   // not for newer compiler versions
+    // this version seems to be better vectorizable on the mic
+    for (i=0; i<n; ++i)
+        ans += av[i]*bv[i];
+
+#else // __MIC__
+    typename vector<Integer>::const_iterator a=av.begin(), b=bv.begin();
+
+    if( n >= 16 )
+    {
+        for( i = 0; i < ( n >> 4 ); ++i, a += 16, b +=16 ){
+            ans += a[0] * b[0];
+            ans += a[1] * b[1];
+            ans += a[2] * b[2];
+            ans += a[3] * b[3];
+            ans += a[4] * b[4];
+            ans += a[5] * b[5];
+            ans += a[6] * b[6];
+            ans += a[7] * b[7];
+            ans += a[8] * b[8];
+            ans += a[9] * b[9];
+            ans += a[10] * b[10];
+            ans += a[11] * b[11];
+            ans += a[12] * b[12];
+            ans += a[13] * b[13];
+            ans += a[14] * b[14];
+            ans += a[15] * b[15];
+        }
+
+        n -= i<<4;
+    }
+
+    if( n >= 8)
+    {
+        ans += a[0] * b[0];
+        ans += a[1] * b[1];
+        ans += a[2] * b[2];
+        ans += a[3] * b[3];
+        ans += a[4] * b[4];
+        ans += a[5] * b[5];
+        ans += a[6] * b[6];
+        ans += a[7] * b[7];
+
+        n -= 8;
+        a += 8;
+        b += 8;
+    }
+
+    if( n >= 4)
+    {
+        ans += a[0] * b[0];
+        ans += a[1] * b[1];
+        ans += a[2] * b[2];
+        ans += a[3] * b[3];
+
+        n -= 4;
+        a += 4;
+        b += 4;
+    }
+
+    if( n >= 2)
+    {
+        ans += a[0] * b[0];
+        ans += a[1] * b[1];
+
+        n -= 2;
+        a += 2;
+        b += 2;
+    }
+
+    if(n>0)
+        ans += a[0]*b[0];
+#endif // __MIC__
+        
+    if(!check_range(ans)){
+		#pragma omp atomic
+		GMP_scal_prod++;
+    
+        // cout << "av " << av;
+        // cout << "bv " << bv;   
+        vector<mpz_class> mpz_a(av.size()), mpz_b(bv.size());
+        convert(mpz_a, av);
+        convert(mpz_b, bv);
+        v_scalar_product(mpz_a,mpz_b);
+    }
+        
+    return (ans>=0);
+}
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+bool v_scalar_product_positive(const vector<Integer>& av,const vector<Integer>& bv){
+    //loop stretching ; brings some small speed improvement
+
+    Integer ans = 0;
+    size_t i,n=av.size();
+
+
+#if 0 // #ifdef __MIC__   // not for newer compiler versions
+    // this version seems to be better vectorizable on the mic
+    for (i=0; i<n; ++i)
+        ans += av[i]*bv[i];
+
+#else // __MIC__
+    typename vector<Integer>::const_iterator a=av.begin(), b=bv.begin();
+
+    if( n >= 16 )
+    {
+        for( i = 0; i < ( n >> 4 ); ++i, a += 16, b +=16 ){
+            ans += a[0] * b[0];
+            ans += a[1] * b[1];
+            ans += a[2] * b[2];
+            ans += a[3] * b[3];
+            ans += a[4] * b[4];
+            ans += a[5] * b[5];
+            ans += a[6] * b[6];
+            ans += a[7] * b[7];
+            ans += a[8] * b[8];
+            ans += a[9] * b[9];
+            ans += a[10] * b[10];
+            ans += a[11] * b[11];
+            ans += a[12] * b[12];
+            ans += a[13] * b[13];
+            ans += a[14] * b[14];
+            ans += a[15] * b[15];
+        }
+
+        n -= i<<4;
+    }
+
+    if( n >= 8)
+    {
+        ans += a[0] * b[0];
+        ans += a[1] * b[1];
+        ans += a[2] * b[2];
+        ans += a[3] * b[3];
+        ans += a[4] * b[4];
+        ans += a[5] * b[5];
+        ans += a[6] * b[6];
+        ans += a[7] * b[7];
+
+        n -= 8;
+        a += 8;
+        b += 8;
+    }
+
+    if( n >= 4)
+    {
+        ans += a[0] * b[0];
+        ans += a[1] * b[1];
+        ans += a[2] * b[2];
+        ans += a[3] * b[3];
+
+        n -= 4;
+        a += 4;
+        b += 4;
+    }
+
+    if( n >= 2)
+    {
+        ans += a[0] * b[0];
+        ans += a[1] * b[1];
+
+        n -= 2;
+        a += 2;
+        b += 2;
+    }
+
+    if(n>0)
+        ans += a[0]*b[0];
+#endif // __MIC__
+        
+    if(!check_range(ans)){
+		#pragma omp atomic
+		GMP_scal_prod++;
+    
+        // cout << "av " << av;
+        // cout << "bv " << bv;   
+        vector<mpz_class> mpz_a(av.size()), mpz_b(bv.size());
+        convert(mpz_a, av);
+        convert(mpz_b, bv);
+        v_scalar_product(mpz_a,mpz_b);
+    }
+        
+    return (ans>0);
+}
+
+template<typename Integer>
+Integer v_scalar_product_vectors_unequal_lungth(const vector<Integer>& a,const vector<Integer>& b){
+    size_t n=min(a.size(),b.size());
+    vector<Integer> trunc_a=a;
+    vector<Integer> trunc_b=b;
+    trunc_a.resize(n);
+    trunc_b.resize(n);
+    return v_scalar_product(trunc_a,trunc_b); 
+}
+
+//---------------------------------------------------------------------------
+/*
 template<typename Integer>
 Integer v_scalar_product_unequal_vectors_end(const vector<Integer>& a,const vector<Integer>& b){
     Integer ans = 0;
@@ -147,7 +355,7 @@ Integer v_scalar_product_unequal_vectors_end(const vector<Integer>& a,const vect
     }
     return ans;
 }
-
+*/
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -212,7 +420,6 @@ vector<Integer> v_abs_value(vector<Integer>& v){
     return w;
 }
 
-
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -243,6 +450,20 @@ Integer v_lcm(const vector<Integer>& v){
     return g;
 }
 
+template<typename Integer>
+Integer v_lcm_to(const vector<Integer>& v,const size_t k, const size_t j){
+    assert(k <= j);
+    size_t i;
+    Integer g=1;
+    for (i = k; i <= j; i++) {
+        g = libnormaliz::lcm(g,v[i]);
+        if (g==0) {
+            return 0;
+        }
+    }
+    return g;
+}
+
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -257,6 +478,30 @@ Integer v_make_prime(vector<Integer>& v){
     return g;
 }
 
+nmz_float l1norm(vector<nmz_float>& v){
+    size_t i, size=v.size();
+    nmz_float g=0;
+    for (i = 0; i < size; i++) {
+        if(Iabs(v[i])>nmz_epsilon)
+            g+=Iabs(v[i]);
+        else
+            v[i]=0;
+    }
+    return g;    
+}
+
+template<>
+nmz_float v_make_prime<>(vector<nmz_float>& v){
+    size_t i, size=v.size();
+    nmz_float g=l1norm(v);
+    if (g!=0) {
+        for (i = 0; i < size; i++) {
+            v[i] /= g;
+        }
+    }
+    return g;
+}
+
 
 //---------------------------------------------------------------------------
 
@@ -295,17 +540,6 @@ vector<Integer> v_scalar_mult_mod(const vector<Integer>& v, const Integer& scala
 
 //---------------------------------------------------------------------------
 
-template<typename Integer>
-void v_scalar_division(vector<Integer>& v, const Integer& scalar){
-    size_t i,size=v.size();
-    for (i = 0; i <size; i++) {
-        assert(v[i]%scalar == 0);
-        v[i] /= scalar;
-    }
-}
-
-//---------------------------------------------------------------------------
-
 template<typename Integer>
 void v_reduction_modulo(vector<Integer>& v, const Integer& modulo){
     size_t i,size=v.size();
@@ -471,6 +705,48 @@ bool v_is_zero(const vector<Integer>& v) {
 
 //---------------------------------------------------------------------------
 
+template<typename Integer>
+bool v_is_symmetric(const vector<Integer>& v) {
+    for (size_t i = 0; i < v.size()/2; ++i) {
+        if (v[i] != v[v.size()-1-i]) return false;
+    }
+    return true;
+}
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+bool v_is_nonnegative(const vector<Integer>& v) {
+    for (size_t i = 0; i < v.size(); ++i) {
+        if (v[i] <0) return false;
+    }
+    return true;
+}
+
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+size_t v_nr_negative(const vector<Integer>& v) {
+    size_t tmp=0;
+    for (size_t i = 0; i < v.size(); ++i) {
+        if (v[i] <0) tmp++;
+    }
+    return tmp;
+}
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+bool v_non_negative(const vector<Integer>& v) {
+    for (size_t i = 0; i < v.size(); ++i) {
+        if (v[i] <0) return false;
+    }
+    return true;
+}
+
+//---------------------------------------------------------------------------
+
 template<typename Integer>
 void v_el_trans(const vector<Integer>& av,vector<Integer>& bv, const Integer& F, const size_t& start){
 
@@ -551,17 +827,18 @@ void v_bool_entry_swap(vector<bool>& v, size_t i, size_t j) {
 // computes approximating lattice simplex using the A_n dissection of the unit cube
 // q is a rational vector with the denominator in the FIRST component q[0]
 
+
 template<typename Integer>
-void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& approx, const long k){
+void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& approx, const long approx_level){
 	
 	//cout << "approximate the point " << q;
     long dim=q.size();
-    long l=k;
-    //if (k>q[0]) l=q[0]; // approximating on level q[0](=grading) is the best we can do
+    long l = approx_level;
+    //if (approx_level>q[0]) l=q[0]; // approximating on level q[0](=grading) is the best we can do
     // TODO in this case, skip the rest and just approximate on q[0]
     Matrix<Integer> quot =  Matrix<Integer>(l,dim);
     Matrix<Integer> remain=Matrix<Integer>(l,dim);
-    for(long j=0;j<l;j++){
+    for(long j=0;j<approx_level;j++){
 	    for(long i=0;i<dim;++i){
 	        quot[j][i]=(q[i]*(j+1))/q[0];          // write q[i]=quot*q[0]+remain
 	        //quot[j][0] = 1;
@@ -595,7 +872,7 @@ void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& appro
 		best_remain[i].first = remain[best_level][i];
 		best_remain[i].second = i; // after sorting we must lnow where elements come from
 	}
-	
+
     sort(best_remain.begin(),best_remain.end()); 
     reverse(best_remain.begin(),best_remain.end()); // we sort remain into descending order
     
@@ -670,8 +947,33 @@ vector<Integer> v_random(size_t n, long m){
         result[i]=rand()%(2*m+1)-m;
     return result;    
 }
+template<typename Integer>
+vector<Integer> degrees_hsop(const vector<Integer> gen_degrees,const vector<size_t> heights){
+    vector<Integer> hsop(heights.back());
+    hsop[0]=gen_degrees[0];
+    size_t k=1;
+    while (k<heights.size() && heights[k]>heights[k-1]){
+        hsop[k]=gen_degrees[k];
+        k++;
+    }
+    size_t j=k;
+    for (size_t i=k;i<heights.size();i++){
+            if (heights[i]>heights[i-1]){
+                hsop[j]=v_lcm_to(gen_degrees,k,i);
+                j++;
+            }
+    }
+    return hsop;
+}
 
 
+template bool v_is_nonnegative<long>(const vector<long>&);
+template bool v_is_nonnegative<long long>(const vector<long long>&);
+template bool v_is_nonnegative<mpz_class>(const vector<mpz_class>&);
+
+template bool v_is_symmetric<long>(const vector<long>&);
+template bool v_is_symmetric<long long>(const vector<long long>&);
+template bool v_is_symmetric<mpz_class>(const vector<mpz_class>&);
 
 template long      v_make_prime(vector<long     >&);
 template long long v_make_prime(vector<long long>&);
@@ -681,4 +983,8 @@ template void v_add_result<long     >(vector<long     >&, size_t, const vector<l
 template void v_add_result<long long>(vector<long long>&, size_t, const vector<long long>&, const vector<long long>&);
 template void v_add_result<mpz_class>(vector<mpz_class>&, size_t, const vector<mpz_class>&, const vector<mpz_class>&);
 
+template long v_scalar_product(const vector<long>& a,const vector<long>& b);
+template long long v_scalar_product(const vector<long long>& a,const vector<long long>& b);
+template mpz_class v_scalar_product(const vector<mpz_class>& a,const vector<mpz_class>& b);
+
 } // end namespace libnormaliz
